@@ -25,13 +25,14 @@ import java.nio.charset.Charset;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
+import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
 
 import javax.servlet.ReadListener;
 
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.coyote.ActionCode;
+import org.apache.coyote.Request;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.buf.B2CConverter;
@@ -41,13 +42,15 @@ import org.apache.tomcat.util.net.ApplicationBufferHandler;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * The buffer used by Tomcat request. This is a derivative of the Tomcat 3.3 OutputBuffer, adapted to handle input
- * instead of output. This allows complete recycling of the facade objects (the ServletInputStream and the
+ * The buffer used by Tomcat request. This is a derivative of the Tomcat 3.3
+ * OutputBuffer, adapted to handle input instead of output. This allows
+ * complete recycling of the facade objects (the ServletInputStream and the
  * BufferedReader).
  *
  * @author Remy Maucherat
  */
-public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, ApplicationBufferHandler {
+public class InputBuffer extends Reader
+    implements ByteChunk.ByteInputChannel, ApplicationBufferHandler {
 
     /**
      * The string manager for this package.
@@ -68,7 +71,7 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
     /**
      * Encoder cache.
      */
-    private static final ConcurrentMap<Charset,SynchronizedStack<B2CConverter>> encoders = new ConcurrentHashMap<>();
+    private static final Map<Charset, SynchronizedStack<B2CConverter>> encoders = new ConcurrentHashMap<>();
 
     // ----------------------------------------------------- Instance Variables
 
@@ -96,12 +99,6 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
     private boolean closed = false;
 
 
-    /**
-     * Encoding to use.
-     */
-    private String enc;
-
-
     /**
      * Current byte to char converter.
      */
@@ -111,7 +108,7 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
     /**
      * Associated Coyote request.
      */
-    private org.apache.coyote.Request coyoteRequest;
+    private Request coyoteRequest;
 
 
     /**
@@ -170,7 +167,7 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
      *
      * @param coyoteRequest Associated Coyote request
      */
-    public void setRequest(org.apache.coyote.Request coyoteRequest) {
+    public void setRequest(Request coyoteRequest) {
         this.coyoteRequest = coyoteRequest;
     }
 
@@ -201,8 +198,6 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
             encoders.get(conv.getCharset()).push(conv);
             conv = null;
         }
-
-        enc = null;
     }
 
 
@@ -220,7 +215,8 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
     public int available() {
         int available = availableInThisBuffer();
         if (available == 0) {
-            coyoteRequest.action(ActionCode.AVAILABLE, Boolean.valueOf(coyoteRequest.getReadListener() != null));
+            coyoteRequest.action(ActionCode.AVAILABLE,
+                    Boolean.valueOf(coyoteRequest.getReadListener() != null));
             available = (coyoteRequest.getAvailable() > 0) ? 1 : 0;
         }
         return available;
@@ -308,6 +304,9 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
         if (closed) {
             return -1;
         }
+        if (coyoteRequest == null) {
+            return -1;
+        }
 
         if (state == INITIAL_STATE) {
             state = BYTE_STATE;
@@ -347,14 +346,14 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
 
 
     /**
-     * Transfers bytes from the buffer to the specified ByteBuffer. After the operation the position of the ByteBuffer
-     * will be returned to the one before the operation, the limit will be the position incremented by the number of the
-     * transferred bytes.
+     * Transfers bytes from the buffer to the specified ByteBuffer. After the
+     * operation the position of the ByteBuffer will be returned to the one
+     * before the operation, the limit will be the position incremented by
+     * the number of the transferred bytes.
      *
      * @param to the ByteBuffer into which bytes are to be written.
-     *
-     * @return an integer specifying the actual number of bytes read, or -1 if the end of the stream is reached
-     *
+     * @return an integer specifying the actual number of bytes read, or -1 if
+     *         the end of the stream is reached
      * @throws IOException if an input or output exception has occurred
      */
     public int read(ByteBuffer to) throws IOException {
@@ -375,18 +374,6 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
 
     // ------------------------------------------------- Chars Handling Methods
 
-
-    /**
-     * @param s New encoding value
-     *
-     * @deprecated This method will be removed in Tomcat 9.0.x
-     */
-    @Deprecated
-    public void setEncoding(String s) {
-        enc = s;
-    }
-
-
     public int realReadChars() throws IOException {
         checkConverter();
 
@@ -547,14 +534,13 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
             return;
         }
 
-        Charset charset = coyoteRequest.getCharset();
+        Charset charset = null;
+        if (coyoteRequest != null) {
+            charset = coyoteRequest.getCharset();
+        }
 
         if (charset == null) {
-            if (enc == null) {
-                charset = org.apache.coyote.Constants.DEFAULT_BODY_CHARSET;
-            } else {
-                charset = B2CConverter.getCharset(enc);
-            }
+            charset = org.apache.coyote.Constants.DEFAULT_BODY_CHARSET;
         }
 
         SynchronizedStack<B2CConverter> stack = encoders.get(charset);
@@ -571,16 +557,10 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
     }
 
 
-    private static B2CConverter createConverter(final Charset charset) throws IOException {
+    private static B2CConverter createConverter(Charset charset) throws IOException {
         if (SecurityUtil.isPackageProtectionEnabled()) {
             try {
-                return AccessController.doPrivileged(new PrivilegedExceptionAction<B2CConverter>() {
-
-                    @Override
-                    public B2CConverter run() throws IOException {
-                        return new B2CConverter(charset);
-                    }
-                });
+                return AccessController.doPrivileged(new PrivilegedCreateConverter(charset));
             } catch (PrivilegedActionException ex) {
                 Exception e = ex.getException();
                 if (e instanceof IOException) {
@@ -640,17 +620,17 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
 
     private void makeSpace(int count) {
         int desiredSize = cb.limit() + count;
-        if (desiredSize > readLimit) {
+        if(desiredSize > readLimit) {
             desiredSize = readLimit;
         }
 
-        if (desiredSize <= cb.capacity()) {
+        if(desiredSize <= cb.capacity()) {
             return;
         }
 
         int newSize = 2 * cb.capacity();
-        if (desiredSize >= newSize) {
-            newSize = 2 * cb.capacity() + count;
+        if(desiredSize >= newSize) {
+            newSize= 2 * cb.capacity() + count;
         }
 
         if (newSize > readLimit) {
@@ -666,4 +646,20 @@ public class InputBuffer extends Reader implements ByteChunk.ByteInputChannel, A
         cb = tmp;
         tmp = null;
     }
+
+
+    private static class PrivilegedCreateConverter
+            implements PrivilegedExceptionAction<B2CConverter> {
+
+        private final Charset charset;
+
+        PrivilegedCreateConverter(Charset charset) {
+            this.charset = charset;
+        }
+
+        @Override
+        public B2CConverter run() throws IOException {
+            return new B2CConverter(charset);
+        }
+    }
 }
