@@ -34,20 +34,21 @@ import javax.servlet.ServletInputStream;
 import javax.servlet.ServletRequest;
 import javax.servlet.ServletResponse;
 import javax.servlet.http.Cookie;
+import javax.servlet.http.HttpServletMapping;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 import javax.servlet.http.HttpUpgradeHandler;
 import javax.servlet.http.Part;
+import javax.servlet.http.PushBuilder;
 
 import org.apache.catalina.Globals;
-import org.apache.catalina.core.ApplicationMappingImpl;
-import org.apache.catalina.core.ApplicationPushBuilder;
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * Facade class that wraps a Coyote request object. All methods are delegated to the wrapped request.
+ * Facade class that wraps a Coyote request object.
+ * All methods are delegated to the wrapped request.
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
@@ -56,7 +57,8 @@ import org.apache.tomcat.util.res.StringManager;
 public class RequestFacade implements HttpServletRequest {
 
 
-    private final class GetAttributePrivilegedAction implements PrivilegedAction<Enumeration<String>> {
+    private final class GetAttributePrivilegedAction
+            implements PrivilegedAction<Enumeration<String>> {
 
         @Override
         public Enumeration<String> run() {
@@ -65,7 +67,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetParameterMapPrivilegedAction implements PrivilegedAction<Map<String,String[]>> {
+    private final class GetParameterMapPrivilegedAction
+            implements PrivilegedAction<Map<String,String[]>> {
 
         @Override
         public Map<String,String[]> run() {
@@ -74,11 +77,12 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetRequestDispatcherPrivilegedAction implements PrivilegedAction<RequestDispatcher> {
+    private final class GetRequestDispatcherPrivilegedAction
+            implements PrivilegedAction<RequestDispatcher> {
 
         private final String path;
 
-        GetRequestDispatcherPrivilegedAction(String path) {
+        GetRequestDispatcherPrivilegedAction(String path){
             this.path = path;
         }
 
@@ -89,11 +93,12 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetParameterPrivilegedAction implements PrivilegedAction<String> {
+    private final class GetParameterPrivilegedAction
+            implements PrivilegedAction<String> {
 
         public String name;
 
-        GetParameterPrivilegedAction(String name) {
+        GetParameterPrivilegedAction(String name){
             this.name = name;
         }
 
@@ -104,7 +109,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetParameterNamesPrivilegedAction implements PrivilegedAction<Enumeration<String>> {
+    private final class GetParameterNamesPrivilegedAction
+            implements PrivilegedAction<Enumeration<String>> {
 
         @Override
         public Enumeration<String> run() {
@@ -113,11 +119,12 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetParameterValuePrivilegedAction implements PrivilegedAction<String[]> {
+    private final class GetParameterValuePrivilegedAction
+            implements PrivilegedAction<String[]> {
 
         public String name;
 
-        GetParameterValuePrivilegedAction(String name) {
+        GetParameterValuePrivilegedAction(String name){
             this.name = name;
         }
 
@@ -128,7 +135,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetCookiesPrivilegedAction implements PrivilegedAction<Cookie[]> {
+    private final class GetCookiesPrivilegedAction
+            implements PrivilegedAction<Cookie[]> {
 
         @Override
         public Cookie[] run() {
@@ -137,7 +145,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetCharacterEncodingPrivilegedAction implements PrivilegedAction<String> {
+    private final class GetCharacterEncodingPrivilegedAction
+            implements PrivilegedAction<String> {
 
         @Override
         public String run() {
@@ -146,11 +155,12 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetHeadersPrivilegedAction implements PrivilegedAction<Enumeration<String>> {
+    private final class GetHeadersPrivilegedAction
+            implements PrivilegedAction<Enumeration<String>> {
 
         private final String name;
 
-        GetHeadersPrivilegedAction(String name) {
+        GetHeadersPrivilegedAction(String name){
             this.name = name;
         }
 
@@ -161,7 +171,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetHeaderNamesPrivilegedAction implements PrivilegedAction<Enumeration<String>> {
+    private final class GetHeaderNamesPrivilegedAction
+            implements PrivilegedAction<Enumeration<String>> {
 
         @Override
         public Enumeration<String> run() {
@@ -170,7 +181,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetLocalePrivilegedAction implements PrivilegedAction<Locale> {
+    private final class GetLocalePrivilegedAction
+            implements PrivilegedAction<Locale> {
 
         @Override
         public Locale run() {
@@ -179,7 +191,8 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
-    private final class GetLocalesPrivilegedAction implements PrivilegedAction<Enumeration<Locale>> {
+    private final class GetLocalesPrivilegedAction
+            implements PrivilegedAction<Enumeration<Locale>> {
 
         @Override
         public Enumeration<Locale> run() {
@@ -187,11 +200,12 @@ public class RequestFacade implements HttpServletRequest {
         }
     }
 
-    private final class GetSessionPrivilegedAction implements PrivilegedAction<HttpSession> {
+    private final class GetSessionPrivilegedAction
+            implements PrivilegedAction<HttpSession> {
 
         private final boolean create;
 
-        GetSessionPrivilegedAction(boolean create) {
+        GetSessionPrivilegedAction(boolean create){
             this.create = create;
         }
 
@@ -205,6 +219,7 @@ public class RequestFacade implements HttpServletRequest {
     private static final StringManager sm = StringManager.getManager(RequestFacade.class);
 
 
+
     /**
      * The wrapped request.
      */
@@ -233,7 +248,8 @@ public class RequestFacade implements HttpServletRequest {
      * Prevent cloning the facade.
      */
     @Override
-    protected Object clone() throws CloneNotSupportedException {
+    protected Object clone()
+        throws CloneNotSupportedException {
         throw new CloneNotSupportedException();
     }
 
@@ -251,8 +267,9 @@ public class RequestFacade implements HttpServletRequest {
     public Enumeration<String> getAttributeNames() {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetAttributePrivilegedAction());
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetAttributePrivilegedAction());
         } else {
             return request.getAttributeNames();
         }
@@ -263,8 +280,9 @@ public class RequestFacade implements HttpServletRequest {
     public String getCharacterEncoding() {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetCharacterEncodingPrivilegedAction());
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetCharacterEncodingPrivilegedAction());
         } else {
             return request.getCharacterEncoding();
         }
@@ -303,8 +321,9 @@ public class RequestFacade implements HttpServletRequest {
     public String getParameter(String name) {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetParameterPrivilegedAction(name));
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetParameterPrivilegedAction(name));
         } else {
             return request.getParameter(name);
         }
@@ -315,8 +334,9 @@ public class RequestFacade implements HttpServletRequest {
     public Enumeration<String> getParameterNames() {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetParameterNamesPrivilegedAction());
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetParameterNamesPrivilegedAction());
         } else {
             return request.getParameterNames();
         }
@@ -330,11 +350,12 @@ public class RequestFacade implements HttpServletRequest {
         String[] ret = null;
 
         /*
-         * Clone the returned array only if there is a security manager in place, so that performance won't suffer in
-         * the non-secure case
+         * Clone the returned array only if there is a security manager
+         * in place, so that performance won't suffer in the non-secure case
          */
-        if (SecurityUtil.isPackageProtectionEnabled()) {
-            ret = AccessController.doPrivileged(new GetParameterValuePrivilegedAction(name));
+        if (SecurityUtil.isPackageProtectionEnabled()){
+            ret = AccessController.doPrivileged(
+                new GetParameterValuePrivilegedAction(name));
             if (ret != null) {
                 ret = ret.clone();
             }
@@ -350,8 +371,9 @@ public class RequestFacade implements HttpServletRequest {
     public Map<String,String[]> getParameterMap() {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetParameterMapPrivilegedAction());
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetParameterMapPrivilegedAction());
         } else {
             return request.getParameterMap();
         }
@@ -425,8 +447,9 @@ public class RequestFacade implements HttpServletRequest {
     public Locale getLocale() {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetLocalePrivilegedAction());
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetLocalePrivilegedAction());
         } else {
             return request.getLocale();
         }
@@ -437,8 +460,9 @@ public class RequestFacade implements HttpServletRequest {
     public Enumeration<Locale> getLocales() {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetLocalesPrivilegedAction());
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetLocalesPrivilegedAction());
         } else {
             return request.getLocales();
         }
@@ -456,8 +480,9 @@ public class RequestFacade implements HttpServletRequest {
     public RequestDispatcher getRequestDispatcher(String path) {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetRequestDispatcherPrivilegedAction(path));
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetRequestDispatcherPrivilegedAction(path));
         } else {
             return request.getRequestDispatcher(path);
         }
@@ -467,7 +492,8 @@ public class RequestFacade implements HttpServletRequest {
     public String getRealPath(String path) {
 
         if (request == null) {
-            throw new IllegalStateException(sm.getString("requestFacade.nullRequest"));
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
         }
 
         return request.getRealPath(path);
@@ -488,11 +514,12 @@ public class RequestFacade implements HttpServletRequest {
         Cookie[] ret = null;
 
         /*
-         * Clone the returned array only if there is a security manager in place, so that performance won't suffer in
-         * the non-secure case
+         * Clone the returned array only if there is a security manager
+         * in place, so that performance won't suffer in the non-secure case
          */
-        if (SecurityUtil.isPackageProtectionEnabled()) {
-            ret = AccessController.doPrivileged(new GetCookiesPrivilegedAction());
+        if (SecurityUtil.isPackageProtectionEnabled()){
+            ret = AccessController.doPrivileged(
+                new GetCookiesPrivilegedAction());
             if (ret != null) {
                 ret = ret.clone();
             }
@@ -522,8 +549,9 @@ public class RequestFacade implements HttpServletRequest {
     public Enumeration<String> getHeaders(String name) {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetHeadersPrivilegedAction(name));
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetHeadersPrivilegedAction(name));
         } else {
             return request.getHeaders(name);
         }
@@ -534,8 +562,9 @@ public class RequestFacade implements HttpServletRequest {
     public Enumeration<String> getHeaderNames() {
         checkFacade();
 
-        if (Globals.IS_SECURITY_ENABLED) {
-            return AccessController.doPrivileged(new GetHeaderNamesPrivilegedAction());
+        if (Globals.IS_SECURITY_ENABLED){
+            return AccessController.doPrivileged(
+                new GetHeaderNamesPrivilegedAction());
         } else {
             return request.getHeaderNames();
         }
@@ -549,6 +578,13 @@ public class RequestFacade implements HttpServletRequest {
     }
 
 
+    @Override
+    public HttpServletMapping getHttpServletMapping() {
+        checkFacade();
+        return request.getHttpServletMapping();
+    }
+
+
     @Override
     public String getMethod() {
         checkFacade();
@@ -637,8 +673,9 @@ public class RequestFacade implements HttpServletRequest {
     public HttpSession getSession(boolean create) {
         checkFacade();
 
-        if (SecurityUtil.isPackageProtectionEnabled()) {
-            return AccessController.doPrivileged(new GetSessionPrivilegedAction(create));
+        if (SecurityUtil.isPackageProtectionEnabled()){
+            return AccessController.
+                doPrivileged(new GetSessionPrivilegedAction(create));
         } else {
             return request.getSession(create);
         }
@@ -646,6 +683,7 @@ public class RequestFacade implements HttpServletRequest {
 
     @Override
     public HttpSession getSession() {
+        checkFacade();
         return getSession(true);
     }
 
@@ -680,7 +718,8 @@ public class RequestFacade implements HttpServletRequest {
     public boolean isRequestedSessionIdFromUrl() {
 
         if (request == null) {
-            throw new IllegalStateException(sm.getString("requestFacade.nullRequest"));
+            throw new IllegalStateException(
+                            sm.getString("requestFacade.nullRequest"));
         }
 
         return request.isRequestedSessionIdFromURL();
@@ -810,28 +849,37 @@ public class RequestFacade implements HttpServletRequest {
 
 
     @Override
-    public <T extends HttpUpgradeHandler> T upgrade(Class<T> httpUpgradeHandlerClass)
-            throws IOException, ServletException {
+    public <T extends HttpUpgradeHandler> T upgrade(
+            Class<T> httpUpgradeHandlerClass) throws java.io.IOException, ServletException {
         checkFacade();
         return request.upgrade(httpUpgradeHandlerClass);
     }
 
 
-    public ApplicationMappingImpl getHttpServletMapping() {
+    @Override
+    public PushBuilder newPushBuilder() {
         checkFacade();
-        return request.getHttpServletMapping();
+        return request.newPushBuilder();
     }
 
 
-    public ApplicationPushBuilder newPushBuilder(javax.servlet.http.HttpServletRequest request) {
+    public PushBuilder newPushBuilder(HttpServletRequest request) {
         checkFacade();
         return this.request.newPushBuilder(request);
     }
 
 
-    public ApplicationPushBuilder newPushBuilder() {
+    @Override
+    public boolean isTrailerFieldsReady() {
         checkFacade();
-        return request.newPushBuilder();
+        return request.isTrailerFieldsReady();
+    }
+
+
+    @Override
+    public Map<String, String> getTrailerFields() {
+        checkFacade();
+        return request.getTrailerFields();
     }
 
 
