@@ -63,9 +63,8 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     private static final String FORK_JOIN_POOL_THREAD_FACTORY_PROPERTY = "java.util.concurrent.ForkJoinPool.common.threadFactory";
     /**
      * Protect against the memory leak caused when the first call to <code>sun.awt.AppContext.getAppContext()</code> is
-     * triggered by a web application. Defaults to <code>false</code> since
-     * {@link java.beans.Introspector#flushCaches()} no longer uses AppContext from 1.7.0_02 onwards. Also, from
-     * 1.7.0_25 onwards, calling this method requires a graphical environment and starts an AWT thread.
+     * triggered by a web application. Defaults to <code>false</code> since Tomcat code no longer triggers this although
+     * application code may.
      */
     private boolean appContextProtection = false;
 
@@ -95,6 +94,8 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
      * Protect against the memory leak caused when the first call to <code>sun.misc.GC.requestLatency(long)</code> is
      * triggered by a web application. This first call will start a GC Daemon thread with the thread's context class
      * loader configured to be the web application class loader. Defaults to <code>true</code>.
+     *
+     * @see "http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8157570"
      */
     private boolean gcDaemonProtection = true;
 
@@ -106,36 +107,6 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
         this.gcDaemonProtection = gcDaemonProtection;
     }
 
-    /**
-     * Protect against the memory leak caused when the first call to <code>javax.security.auth.Policy</code> is
-     * triggered by a web application. This first call populate a static variable with a reference to the context class
-     * loader. Defaults to <code>true</code>.
-     */
-    private boolean securityPolicyProtection = true;
-
-    public boolean isSecurityPolicyProtection() {
-        return securityPolicyProtection;
-    }
-
-    public void setSecurityPolicyProtection(boolean securityPolicyProtection) {
-        this.securityPolicyProtection = securityPolicyProtection;
-    }
-
-    /**
-     * Protects against the memory leak caused when the first call to
-     * <code>javax.security.auth.login.Configuration</code> is triggered by a web application. This first call populate
-     * a static variable with a reference to the context class loader. Defaults to <code>true</code>.
-     */
-    private boolean securityLoginConfigurationProtection = true;
-
-    public boolean isSecurityLoginConfigurationProtection() {
-        return securityLoginConfigurationProtection;
-    }
-
-    public void setSecurityLoginConfigurationProtection(boolean securityLoginConfigurationProtection) {
-        this.securityLoginConfigurationProtection = securityLoginConfigurationProtection;
-    }
-
     /**
      * Protect against the memory leak, when the initialization of the Java Cryptography Architecture is triggered by
      * initializing a MessageDigest during web application deployment. This will occasionally start a Token Poller
@@ -186,6 +157,8 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
      * property <code>com.sun.jndi.ldap.connect.pool.timeout</code> is greater than 0. That thread inherits the context
      * class loader of the current thread, so that there may be a web application class loader leak if the web app is
      * the first to use <code>LdapPoolManager</code>.
+     *
+     * @see "http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8156824"
      */
     private boolean ldapPoolProtection = true;
 
@@ -213,8 +186,10 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     }
 
     /**
-     * {@link ForkJoinPool} creates a thread pool that, by default, creates threads that retain references to the thread
-     * context class loader.
+     * {@link ForkJoinPool#commonPool()} creates a thread pool that, by default, creates threads that retain references
+     * to the thread context class loader.
+     *
+     * @see "http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8172726"
      */
     private boolean forkJoinCommonPoolProtection = true;
 
@@ -262,35 +237,33 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 DriverManager.getDrivers();
             }
 
-            Thread currentThread = Thread.currentThread();
-            ClassLoader loader = currentThread.getContextClassLoader();
+            ClassLoader loader = Thread.currentThread().getContextClassLoader();
 
             try {
                 // Use the system classloader as the victim for all this
                 // ClassLoader pinning we're about to do.
-                currentThread.setContextClassLoader(ClassLoader.getSystemClassLoader());
+                Thread.currentThread().setContextClassLoader(ClassLoader.getSystemClassLoader());
 
                 /*
                  * Several components end up calling: sun.awt.AppContext.getAppContext()
                  *
                  * Those libraries / components known to trigger memory leaks due to eventual calls to getAppContext()
-                 * are: - Google Web Toolkit via its use of javax.imageio - Tomcat via its use of
-                 * java.beans.Introspector.flushCaches() in 1.7.0 to 1.7.0_01. From 1.7.0_02 onwards use of AppContext
-                 * by Introspector.flushCaches() was replaced with ThreadGroupContext - others TBD
+                 * are: - Google Web Toolkit via its use of javax.imageio - Batik - others TBD
                  *
-                 * From 1.7.0_25 onwards, a call to sun.awt.AppContext.getAppContext() results in a thread being started
-                 * named AWT-AppKit that requires a graphic environment to be available.
+                 * Note that a call to sun.awt.AppContext.getAppContext() results in a thread being started named
+                 * AWT-AppKit that requires a graphical environment to be available.
                  */
 
                 // Trigger a call to sun.awt.AppContext.getAppContext(). This
                 // will pin the system class loader in memory but that shouldn't
                 // be an issue.
-                if (appContextProtection && !JreCompat.isJre8Available()) {
+                if (appContextProtection) {
                     ImageIO.getCacheDirectory();
                 }
 
                 // Trigger the creation of the AWT (AWT-Windows, AWT-XAWT,
-                // etc.) thread
+                // etc.) thread.
+                // Note this issue is fixed in Java 8 update 05 onwards.
                 if (awtThreadProtection && !JreCompat.isJre9Available()) {
                     java.awt.Toolkit.getDefaultToolkit();
                 }
@@ -304,6 +277,7 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                  *
                  * Note: Long.MAX_VALUE is a special case that causes the thread to terminate
                  *
+                 * Fixed in Java 9 onwards (from early access build 130)
                  */
                 if (gcDaemonProtection && !JreCompat.isJre9Available()) {
                     try {
@@ -325,46 +299,6 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                     }
                 }
 
-                /*
-                 * Calling getPolicy retains a static reference to the context class loader.
-                 */
-                if (securityPolicyProtection && !JreCompat.isJre8Available()) {
-                    try {
-                        // Policy.getPolicy();
-                        Class<?> policyClass = Class.forName("javax.security.auth.Policy");
-                        Method method = policyClass.getMethod("getPolicy");
-                        method.invoke(null);
-                    } catch (ClassNotFoundException e) {
-                        // Ignore. The class is deprecated.
-                    } catch (SecurityException e) {
-                        // Ignore. Don't need call to getPolicy() to be
-                        // successful, just need to trigger static initializer.
-                    } catch (NoSuchMethodException e) {
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"), e);
-                    } catch (IllegalArgumentException e) {
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"), e);
-                    } catch (IllegalAccessException e) {
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"), e);
-                    } catch (InvocationTargetException e) {
-                        ExceptionUtils.handleThrowable(e.getCause());
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"), e);
-                    }
-                }
-
-
-                /*
-                 * Initializing javax.security.auth.login.Configuration retains a static reference to the context class
-                 * loader.
-                 */
-                if (securityLoginConfigurationProtection && !JreCompat.isJre8Available()) {
-                    try {
-                        Class.forName("javax.security.auth.login.Configuration", true,
-                                ClassLoader.getSystemClassLoader());
-                    } catch (ClassNotFoundException e) {
-                        // Ignore
-                    }
-                }
-
                 /*
                  * Creating a MessageDigest during web application startup initializes the Java Cryptography
                  * Architecture. Under certain conditions this starts a Token poller thread with TCCL equal to the web
@@ -386,6 +320,8 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                  * Those libraries/components known to trigger this issue include: - log4j versions 1.2.15 and earlier -
                  * javax.xml.bind.JAXBContext.newInstance()
                  *
+                 * https://bugs.openjdk.java.net/browse/JDK-8163449
+                 *
                  * Java 9 onwards disables caching for JAR URLConnections Java 8 and earlier disables caching for all
                  * URLConnections
                  */
@@ -404,11 +340,12 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                  */
                 if (xmlParsingProtection && !JreCompat.isJre9Available()) {
                     // There are two known issues with XML parsing that affect
-                    // Java 7+. The issues both relate to cached Exception
+                    // Java 8+. The issues both relate to cached Exception
                     // instances that retain a link to the TCCL via the
                     // backtrace field. Note that YourKit only shows this field
                     // when using the HPROF format memory snapshots.
                     // https://bz.apache.org/bugzilla/show_bug.cgi?id=58486
+                    // https://bugs.openjdk.java.net/browse/JDK-8146961
                     DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                     try {
                         DocumentBuilder documentBuilder = factory.newDocumentBuilder();
@@ -426,6 +363,9 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                     }
                 }
 
+                /*
+                 * Fixed in Java 9 onwards (from early access build 130)
+                 */
                 if (ldapPoolProtection && !JreCompat.isJre9Available()) {
                     try {
                         Class.forName("com.sun.jndi.ldap.LdapPoolManager");
@@ -439,10 +379,9 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 }
 
                 /*
-                 * Present in Java 7 onwards Work-around only available in Java 8. Fixed in Java 9 (from early access
-                 * build 156)
+                 * Present in Java 7 onwards Fixed in Java 9 (from early access build 156)
                  */
-                if (forkJoinCommonPoolProtection && JreCompat.isJre8Available() && !JreCompat.isJre9Available()) {
+                if (forkJoinCommonPoolProtection && !JreCompat.isJre9Available()) {
                     // Don't override any explicitly set property
                     if (System.getProperty(FORK_JOIN_POOL_THREAD_FACTORY_PROPERTY) == null) {
                         System.setProperty(FORK_JOIN_POOL_THREAD_FACTORY_PROPERTY,
@@ -464,7 +403,7 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 }
 
             } finally {
-                currentThread.setContextClassLoader(loader);
+                Thread.currentThread().setContextClassLoader(loader);
             }
         }
     }
