@@ -29,6 +29,7 @@ import javax.servlet.http.HttpServletResponse;
 
 import org.apache.coyote.AbstractProcessor;
 import org.apache.coyote.ActionCode;
+import org.apache.coyote.Adapter;
 import org.apache.coyote.ContinueResponseTiming;
 import org.apache.coyote.ErrorState;
 import org.apache.coyote.Request;
@@ -51,13 +52,11 @@ import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
-import org.apache.tomcat.util.buf.StringUtils;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.parser.HttpParser;
 import org.apache.tomcat.util.http.parser.TokenList;
 import org.apache.tomcat.util.log.UserDataHelper;
-import org.apache.tomcat.util.net.AbstractEndpoint;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.ApplicationBufferHandler;
 import org.apache.tomcat.util.net.SSLSupport;
@@ -83,21 +82,21 @@ public class Http11Processor extends AbstractProcessor {
     /**
      * Input.
      */
-    protected final Http11InputBuffer inputBuffer;
+    private final Http11InputBuffer inputBuffer;
 
 
     /**
      * Output.
      */
-    protected final Http11OutputBuffer outputBuffer;
+    private final Http11OutputBuffer outputBuffer;
 
 
     private final HttpParser httpParser;
 
 
     /**
-     * Tracks how many internal filters are in the filter library so they are skipped when looking for pluggable
-     * filters.
+     * Tracks how many internal filters are in the filter library so they
+     * are skipped when looking for pluggable filters.
      */
     private int pluggableFilterIndex = Integer.MAX_VALUE;
 
@@ -105,93 +104,65 @@ public class Http11Processor extends AbstractProcessor {
     /**
      * Keep-alive.
      */
-    protected volatile boolean keepAlive = true;
+    private volatile boolean keepAlive = true;
 
 
     /**
-     * Flag used to indicate that the socket should be kept open (e.g. for keep alive or send file).
+     * Flag used to indicate that the socket should be kept open (e.g. for keep
+     * alive or send file).
      */
-    protected volatile boolean openSocket = false;
+    private volatile boolean openSocket = false;
 
 
     /**
      * Flag that indicates if the request headers have been completely read.
      */
-    protected volatile boolean readComplete = true;
+    private volatile boolean readComplete = true;
 
     /**
      * HTTP/1.1 flag.
      */
-    protected boolean http11 = true;
+    private boolean http11 = true;
 
 
     /**
      * HTTP/0.9 flag.
      */
-    protected boolean http09 = false;
+    private boolean http09 = false;
 
 
     /**
-     * Content delimiter for the request (if false, the connection will be closed at the end of the request).
+     * Content delimiter for the request (if false, the connection will
+     * be closed at the end of the request).
      */
-    protected boolean contentDelimitation = true;
+    private boolean contentDelimitation = true;
 
 
     /**
-     * Regular expression that defines the restricted user agents.
+     * Instance of the new protocol to use after the HTTP connection has been
+     * upgraded.
      */
-    protected Pattern restrictedUserAgents = null;
-
-
-    /**
-     * Maximum number of Keep-Alive requests to honor.
-     */
-    protected int maxKeepAliveRequests = -1;
-
-
-    /**
-     * Maximum timeout on uploads. 5 minutes as in Apache HTTPD server.
-     */
-    protected int connectionUploadTimeout = 300000;
-
-
-    /**
-     * Flag to disable setting a different time-out on uploads.
-     */
-    protected boolean disableUploadTimeout = false;
-
-
-    /**
-     * Max saved post size.
-     */
-    protected int maxSavePostSize = 4 * 1024;
-
-
-    /**
-     * Instance of the new protocol to use after the HTTP connection has been upgraded.
-     */
-    protected UpgradeToken upgradeToken = null;
+    private UpgradeToken upgradeToken = null;
 
 
     /**
      * Sendfile data.
      */
-    protected SendfileDataBase sendfileData = null;
+    private SendfileDataBase sendfileData = null;
 
 
-    @SuppressWarnings("deprecation")
-    public Http11Processor(AbstractHttp11Protocol<?> protocol, AbstractEndpoint<?, ?> endpoint) {
-        super(endpoint);
+    public Http11Processor(AbstractHttp11Protocol<?> protocol, Adapter adapter) {
+        super(adapter);
         this.protocol = protocol;
 
-        httpParser = new HttpParser(protocol.getRelaxedPathChars(), protocol.getRelaxedQueryChars());
+        httpParser = new HttpParser(protocol.getRelaxedPathChars(),
+                protocol.getRelaxedQueryChars());
 
         inputBuffer = new Http11InputBuffer(request, protocol.getMaxHttpRequestHeaderSize(),
                 protocol.getRejectIllegalHeader(), httpParser);
         request.setInputBuffer(inputBuffer);
 
-        outputBuffer = new Http11OutputBuffer(response, protocol.getMaxHttpResponseHeaderSize(),
-                protocol.getSendReasonPhrase());
+        outputBuffer = new Http11OutputBuffer(response, protocol.getMaxHttpResponseHeaderSize());
         response.setOutputBuffer(outputBuffer);
 
         // Create and add the identity filters.
@@ -199,9 +170,9 @@ public class Http11Processor extends AbstractProcessor {
         outputBuffer.addFilter(new IdentityOutputFilter());
 
         // Create and add the chunked filters.
-        inputBuffer.addFilter(
-                new ChunkedInputFilter(protocol.getMaxTrailerSize(), protocol.getAllowedTrailerHeadersInternal(),
-                        protocol.getMaxExtensionSize(), protocol.getMaxSwallowSize()));
+        inputBuffer.addFilter(new ChunkedInputFilter(protocol.getMaxTrailerSize(),
+                protocol.getAllowedTrailerHeadersInternal(), protocol.getMaxExtensionSize(),
+                protocol.getMaxSwallowSize()));
         outputBuffer.addFilter(new ChunkedOutputFilter());
 
         // Create and add the void filters.
@@ -212,7 +183,7 @@ public class Http11Processor extends AbstractProcessor {
         inputBuffer.addFilter(new BufferedInputFilter(protocol.getMaxSwallowSize()));
 
         // Create and add the gzip filters.
-        // inputBuffer.addFilter(new GzipInputFilter());
+        //inputBuffer.addFilter(new GzipInputFilter());
         outputBuffer.addFilter(new GzipOutputFilter());
 
         pluggableFilterIndex = inputBuffer.getFilters().length;
@@ -220,217 +191,24 @@ public class Http11Processor extends AbstractProcessor {
 
 
     /**
-     * Set compression level.
-     *
-     * @param compression One of <code>on</code>, <code>force</code>, <code>off</code> or the minimum compression size
-     *                        in bytes which implies <code>on</code>
-     *
-     * @deprecated Use {@link Http11Protocol#setCompression(String)}
-     */
-    @Deprecated
-    public void setCompression(String compression) {
-        protocol.setCompression(compression);
-    }
-
-    /**
-     * Set Minimum size to trigger compression.
-     *
-     * @param compressionMinSize The minimum content length required for compression in bytes
-     *
-     * @deprecated Use {@link Http11Protocol#setCompressionMinSize(int)}
-     */
-    @Deprecated
-    public void setCompressionMinSize(int compressionMinSize) {
-        protocol.setCompressionMinSize(compressionMinSize);
-    }
-
-
-    /**
-     * Set no compression user agent pattern. Regular expression as supported by {@link Pattern}. e.g.:
-     * <code>gorilla|desesplorer|tigrus</code>.
-     *
-     * @param noCompressionUserAgents The regular expression for user agent strings for which compression should not be
-     *                                    applied
-     *
-     * @deprecated Use {@link Http11Protocol#setNoCompressionUserAgents(String)}
-     */
-    @Deprecated
-    public void setNoCompressionUserAgents(String noCompressionUserAgents) {
-        protocol.setNoCompressionUserAgents(noCompressionUserAgents);
-    }
-
-
-    /**
-     * @param compressibleMimeTypes See {@link Http11Processor#setCompressibleMimeTypes(String[])}
-     *
-     * @deprecated Use {@link Http11Protocol#setCompressibleMimeType(String)}
-     */
-    @Deprecated
-    public void setCompressableMimeTypes(String[] compressibleMimeTypes) {
-        setCompressibleMimeTypes(compressibleMimeTypes);
-    }
-
-
-    /**
-     * Set compressible mime-type list.
-     *
-     * @param compressibleMimeTypes MIME types for which compression should be enabled
-     *
-     * @deprecated Use {@link Http11Protocol#setCompressibleMimeType(String)}
-     */
-    @Deprecated
-    public void setCompressibleMimeTypes(String[] compressibleMimeTypes) {
-        // Conversion from array to comma separated string and back to array in
-        // CompressionConfig isn't ideal but it is better than adding a direct
-        // setter only to immediately deprecate it as it doesn't exist in 9.0.x
-        // given that Tomcat doesn't call this method in normal usage.
-        protocol.setCompressibleMimeType(StringUtils.join(compressibleMimeTypes));
-    }
-
-
-    /**
-     * Return compression level.
-     *
-     * @return The current compression level in string form (off/on/force)
-     *
-     * @deprecated Use {@link Http11Protocol#getCompression()}
-     */
-    @Deprecated
-    public String getCompression() {
-        return protocol.getCompression();
-    }
-
-
-    /**
-     * Set restricted user agent list (which will downgrade the connector to HTTP/1.0 mode). Regular expression as
-     * supported by {@link Pattern}.
-     *
-     * @param restrictedUserAgents The regular expression as supported by {@link Pattern} for the user agents e.g.
-     *                                 "gorilla|desesplorer|tigrus"
-     */
-    public void setRestrictedUserAgents(String restrictedUserAgents) {
-        if (restrictedUserAgents == null || restrictedUserAgents.length() == 0) {
-            this.restrictedUserAgents = null;
-        } else {
-            this.restrictedUserAgents = Pattern.compile(restrictedUserAgents);
-        }
-    }
-
-
-    /**
-     * Set the maximum number of Keep-Alive requests to allow. This is to safeguard from DoS attacks. Setting to a
-     * negative value disables the limit.
-     *
-     * @param mkar The new maximum number of Keep-Alive requests allowed
-     */
-    public void setMaxKeepAliveRequests(int mkar) {
-        maxKeepAliveRequests = mkar;
-    }
-
-
-    /**
-     * Get the maximum number of Keep-Alive requests allowed. A negative value means there is no limit.
-     *
-     * @return the number of Keep-Alive requests that we will allow.
-     */
-    public int getMaxKeepAliveRequests() {
-        return maxKeepAliveRequests;
-    }
-
-
-    /**
-     * Set the maximum size of a POST which will be buffered in SSL mode. When a POST is received where the security
-     * constraints require a client certificate, the POST body needs to be buffered while an SSL handshake takes place
-     * to obtain the certificate.
-     *
-     * @param msps The maximum size POST body to buffer in bytes
-     */
-    public void setMaxSavePostSize(int msps) {
-        maxSavePostSize = msps;
-    }
-
-
-    /**
-     * Return the maximum size of a POST which will be buffered in SSL mode.
-     *
-     * @return The size in bytes
-     */
-    public int getMaxSavePostSize() {
-        return maxSavePostSize;
-    }
-
-
-    /**
-     * Set the flag to control whether a separate connection timeout is used during upload of a request body.
-     *
-     * @param isDisabled {@code true} if the separate upload timeout should be disabled
-     */
-    public void setDisableUploadTimeout(boolean isDisabled) {
-        disableUploadTimeout = isDisabled;
-    }
-
-    /**
-     * Get the flag that controls upload time-outs.
-     *
-     * @return {@code true} if the separate upload timeout is disabled
-     */
-    public boolean getDisableUploadTimeout() {
-        return disableUploadTimeout;
-    }
-
-    /**
-     * Set the upload timeout.
-     *
-     * @param timeout Upload timeout in milliseconds
-     */
-    public void setConnectionUploadTimeout(int timeout) {
-        connectionUploadTimeout = timeout;
-    }
-
-    /**
-     * Get the upload timeout.
-     *
-     * @return Upload timeout in milliseconds
-     */
-    public int getConnectionUploadTimeout() {
-        return connectionUploadTimeout;
-    }
-
-
-    /**
-     * Set the server header name.
-     *
-     * @param server The new value to use for the server header
-     *
-     * @deprecated Use {@link Http11Protocol#setServer(String)}
-     */
-    @Deprecated
-    public void setServer(String server) {
-        protocol.setServer(server);
-    }
-
-
-    @Deprecated
-    public void setServerRemoveAppProvidedValues(boolean serverRemoveAppProvidedValues) {
-        protocol.setServerRemoveAppProvidedValues(serverRemoveAppProvidedValues);
-    }
-
-
-    /**
-     * Determine if we must drop the connection because of the HTTP status code. Use the same list of codes as
-     * Apache/httpd.
+     * Determine if we must drop the connection because of the HTTP status
+     * code.  Use the same list of codes as Apache/httpd.
      */
     private static boolean statusDropsConnection(int status) {
-        return status == 400 /* SC_BAD_REQUEST */ || status == 408 /* SC_REQUEST_TIMEOUT */ ||
-                status == 411 /* SC_LENGTH_REQUIRED */ || status == 413 /* SC_REQUEST_ENTITY_TOO_LARGE */ ||
-                status == 414 /* SC_REQUEST_URI_TOO_LONG */ || status == 500 /* SC_INTERNAL_SERVER_ERROR */ ||
-                status == 503 /* SC_SERVICE_UNAVAILABLE */ || status == 501 /* SC_NOT_IMPLEMENTED */;
+        return status == 400 /* SC_BAD_REQUEST */ ||
+               status == 408 /* SC_REQUEST_TIMEOUT */ ||
+               status == 411 /* SC_LENGTH_REQUIRED */ ||
+               status == 413 /* SC_REQUEST_ENTITY_TOO_LARGE */ ||
+               status == 414 /* SC_REQUEST_URI_TOO_LONG */ ||
+               status == 500 /* SC_INTERNAL_SERVER_ERROR */ ||
+               status == 503 /* SC_SERVICE_UNAVAILABLE */ ||
+               status == 501 /* SC_NOT_IMPLEMENTED */;
     }
 
 
     /**
-     * Add an input filter to the current request. If the encoding is not supported, a 501 response will be returned to
-     * the client.
+     * Add an input filter to the current request. If the encoding is not
+     * supported, a 501 response will be returned to the client.
      */
     private void addInputFilter(InputFilter[] inputFilters, String encodingName) {
         if (contentDelimitation) {
@@ -441,7 +219,7 @@ public class Http11Processor extends AbstractProcessor {
             setErrorState(ErrorState.CLOSE_CLEAN, null);
             if (log.isDebugEnabled()) {
                 log.debug(sm.getString("http11processor.request.prepare") +
-                        " Transfer encoding lists chunked before [" + encodingName + "]");
+                          " Transfer encoding lists chunked before [" + encodingName + "]");
             }
             return;
         }
@@ -462,15 +240,16 @@ public class Http11Processor extends AbstractProcessor {
             response.setStatus(501);
             setErrorState(ErrorState.CLOSE_CLEAN, null);
             if (log.isDebugEnabled()) {
-                log.debug(sm.getString("http11processor.request.prepare") + " Unsupported transfer encoding [" +
-                        encodingName + "]");
+                log.debug(sm.getString("http11processor.request.prepare") +
+                          " Unsupported transfer encoding [" + encodingName + "]");
             }
         }
     }
 
 
     @Override
-    public SocketState service(SocketWrapperBase<?> socketWrapper) throws IOException {
+    public SocketState service(SocketWrapperBase<?> socketWrapper)
+        throws IOException {
         RequestInfo rp = request.getRequestProcessor();
         rp.setStage(org.apache.coyote.Constants.STAGE_PARSE);
 
@@ -485,11 +264,12 @@ public class Http11Processor extends AbstractProcessor {
         SendfileState sendfileState = SendfileState.DONE;
 
         while (!getErrorState().isError() && keepAlive && !isAsync() && upgradeToken == null &&
-                sendfileState == SendfileState.DONE && !endpoint.isPaused()) {
+                sendfileState == SendfileState.DONE && !protocol.isPaused()) {
 
             // Parsing the request header
             try {
-                if (!inputBuffer.parseRequestLine(keptAlive)) {
+                if (!inputBuffer.parseRequestLine(keptAlive, protocol.getConnectionTimeout(),
+                        protocol.getKeepAliveTimeout())) {
                     if (inputBuffer.getParsingRequestLinePhase() == -1) {
                         return SocketState.UPGRADING;
                     } else if (handleIncompleteRequestLineRead()) {
@@ -502,14 +282,14 @@ public class Http11Processor extends AbstractProcessor {
                 // parse headers.
                 prepareRequestProtocol();
 
-                if (endpoint.isPaused()) {
+                if (protocol.isPaused()) {
                     // 503 - Service unavailable
                     response.setStatus(503);
                     setErrorState(ErrorState.CLOSE_CLEAN, null);
                 } else {
                     keptAlive = true;
                     // Set this every time in case limit has been changed via JMX
-                    request.getMimeHeaders().setLimit(endpoint.getMaxHeaderCount());
+                    request.getMimeHeaders().setLimit(protocol.getMaxHeaderCount());
                     // Don't parse headers for HTTP/0.9
                     if (!http09 && !inputBuffer.parseHeaders()) {
                         // We've read part of the request, don't recycle it
@@ -518,8 +298,8 @@ public class Http11Processor extends AbstractProcessor {
                         readComplete = false;
                         break;
                     }
-                    if (!disableUploadTimeout) {
-                        socketWrapper.setReadTimeout(connectionUploadTimeout);
+                    if (!protocol.getDisableUploadTimeout()) {
+                        socketWrapper.setReadTimeout(protocol.getConnectionUploadTimeout());
                     }
                 }
             } catch (IOException e) {
@@ -574,12 +354,12 @@ public class Http11Processor extends AbstractProcessor {
                             response.setStatus(HttpServletResponse.SC_SWITCHING_PROTOCOLS);
                             response.setHeader("Connection", "Upgrade");
                             response.setHeader("Upgrade", requestedProtocol);
-                            action(ActionCode.CLOSE, null);
+                            action(ActionCode.CLOSE,  null);
                             getAdapter().log(request, response, 0);
 
                             // Continue processing using new protocol
-                            InternalHttpUpgradeHandler upgradeHandler = upgradeProtocol
-                                    .getInternalUpgradeHandler(getAdapter(), upgradeRequest);
+                            InternalHttpUpgradeHandler upgradeHandler =
+                                    upgradeProtocol.getInternalUpgradeHandler(socketWrapper, getAdapter(), upgradeRequest);
                             UpgradeToken upgradeToken = new UpgradeToken(upgradeHandler, null, null, requestedProtocol);
                             action(ActionCode.UPGRADE, upgradeToken);
                             return SocketState.UPGRADING;
@@ -604,9 +384,11 @@ public class Http11Processor extends AbstractProcessor {
                 }
             }
 
+            int maxKeepAliveRequests = protocol.getMaxKeepAliveRequests();
             if (maxKeepAliveRequests == 1) {
                 keepAlive = false;
-            } else if (maxKeepAliveRequests > 0 && socketWrapper.decrementKeepAlive() <= 0) {
+            } else if (maxKeepAliveRequests > 0 &&
+                    socketWrapper.decrementKeepAlive() <= 0) {
                 keepAlive = false;
             }
 
@@ -616,11 +398,11 @@ public class Http11Processor extends AbstractProcessor {
                     rp.setStage(org.apache.coyote.Constants.STAGE_SERVICE);
                     getAdapter().service(request, response);
                     // Handle when the response was committed before a serious
-                    // error occurred. Throwing a ServletException should both
+                    // error occurred.  Throwing a ServletException should both
                     // set the status to 500 and set the errorException.
                     // If we fail here, then the response is likely already
                     // committed, so we can't try and set headers.
-                    if (keepAlive && !getErrorState().isError() && !isAsync() &&
+                    if(keepAlive && !getErrorState().isError() && !isAsync() &&
                             statusDropsConnection(response.getStatus())) {
                         setErrorState(ErrorState.CLOSE_CLEAN, null);
                     }
@@ -672,10 +454,10 @@ public class Http11Processor extends AbstractProcessor {
                 }
             }
 
-            if (!disableUploadTimeout) {
-                int soTimeout = endpoint.getConnectionTimeout();
-                if (soTimeout > 0) {
-                    socketWrapper.setReadTimeout(soTimeout);
+            if (!protocol.getDisableUploadTimeout()) {
+                int connectionTimeout = protocol.getConnectionTimeout();
+                if(connectionTimeout > 0) {
+                    socketWrapper.setReadTimeout(connectionTimeout);
                 } else {
                     socketWrapper.setReadTimeout(0);
                 }
@@ -688,7 +470,7 @@ public class Http11Processor extends AbstractProcessor {
 
         rp.setStage(org.apache.coyote.Constants.STAGE_ENDED);
 
-        if (getErrorState().isError() || (endpoint.isPaused() && !isAsync())) {
+        if (getErrorState().isError() || (protocol.isPaused() && !isAsync())) {
             return SocketState.CLOSED;
         } else if (isAsync()) {
             return SocketState.LONG;
@@ -740,7 +522,7 @@ public class Http11Processor extends AbstractProcessor {
         // Need to read and buffer the request body, if any. RFC 7230 requires
         // that the request is fully read before the upgrade takes place.
         ByteChunk body = new ByteChunk();
-        int maxSavePostSize = this.maxSavePostSize;
+        int maxSavePostSize = protocol.getMaxSavePostSize();
         if (maxSavePostSize != 0) {
             body.setLimit(maxSavePostSize);
             ApplicationBufferHandler buffer = new UpgradeApplicationBufferHandler();
@@ -765,7 +547,7 @@ public class Http11Processor extends AbstractProcessor {
         // Check to see if we have read any of the request line yet
         if (inputBuffer.getParsingRequestLinePhase() > 1) {
             // Started to read request line.
-            if (endpoint.isPaused()) {
+            if (protocol.isPaused()) {
                 // Partially processed the request so need to respond
                 response.setStatus(503);
                 setErrorState(ErrorState.CLOSE_CLEAN, null);
@@ -839,8 +621,8 @@ public class Http11Processor extends AbstractProcessor {
             response.setStatus(505);
             setErrorState(ErrorState.CLOSE_CLEAN, null);
             if (log.isDebugEnabled()) {
-                log.debug(sm.getString("http11processor.request.prepare") + " Unsupported HTTP version \"" +
-                        protocolMB + "\"");
+                log.debug(sm.getString("http11processor.request.prepare")+
+                          " Unsupported HTTP version \""+protocolMB+"\"");
             }
         }
     }
@@ -849,10 +631,9 @@ public class Http11Processor extends AbstractProcessor {
     /**
      * After reading the request headers, we have to setup the request filters.
      */
-    @SuppressWarnings("deprecation")
     private void prepareRequest() throws IOException {
 
-        if (endpoint.isSSLEnabled()) {
+        if (protocol.isSSLEnabled()) {
             request.scheme().setString("https");
         }
 
@@ -875,13 +656,14 @@ public class Http11Processor extends AbstractProcessor {
         }
 
         // Check user-agent header
+        Pattern restrictedUserAgents = protocol.getRestrictedUserAgentsPattern();
         if (restrictedUserAgents != null && (http11 || keepAlive)) {
             MessageBytes userAgentValueMB = headers.getValue("user-agent");
             // Check in the restricted list, and adjust the http11
             // and keepAlive flags accordingly
-            if (userAgentValueMB != null && !userAgentValueMB.isNull()) {
+            if(userAgentValueMB != null && !userAgentValueMB.isNull()) {
                 String userAgentValue = userAgentValueMB.toString();
-                if (restrictedUserAgents != null && restrictedUserAgents.matcher(userAgentValue).matches()) {
+                if (restrictedUserAgents.matcher(userAgentValue).matches()) {
                     http11 = false;
                     keepAlive = false;
                 }
@@ -959,7 +741,8 @@ public class Http11Processor extends AbstractProcessor {
                     if (hostValueMB != null) {
                         // Any host in the request line must be consistent with
                         // the Host header
-                        if (!hostValueMB.getByteChunk().equalsIgnoreCase(uriB, uriBCStart + pos, slashPos - pos)) {
+                        if (!hostValueMB.getByteChunk().equals(
+                                uriB, uriBCStart + pos, slashPos - pos)) {
                             if (protocol.getAllowHostHeaderMismatch()) {
                                 // The requirements of RFC 2616 are being
                                 // applied. If the host header and the request
@@ -1096,7 +879,8 @@ public class Http11Processor extends AbstractProcessor {
 
 
     /**
-     * When committing the response, we have to validate the set of headers, as well as setup the response filters.
+     * When committing the response, we have to validate the set of headers, as
+     * well as setup the response filters.
      */
     @Override
     protected final void prepareResponse() throws IOException {
@@ -1114,9 +898,11 @@ public class Http11Processor extends AbstractProcessor {
         }
 
         int statusCode = response.getStatus();
-        if (statusCode < 200 || statusCode == 204 || statusCode == 205 || statusCode == 304) {
+        if (statusCode < 200 || statusCode == 204 || statusCode == 205 ||
+                statusCode == 304) {
             // No entity body
-            outputBuffer.addActiveFilter(outputFilters[Constants.VOID_FILTER]);
+            outputBuffer.addActiveFilter
+                (outputFilters[Constants.VOID_FILTER]);
             entityBody = false;
             contentDelimitation = true;
             if (statusCode == 205) {
@@ -1128,14 +914,16 @@ public class Http11Processor extends AbstractProcessor {
             }
         }
 
-        if (request.method().equals("HEAD")) {
+        MessageBytes methodMB = request.method();
+        if (methodMB.equals("HEAD")) {
             // No entity body
-            outputBuffer.addActiveFilter(outputFilters[Constants.VOID_FILTER]);
+            outputBuffer.addActiveFilter
+                (outputFilters[Constants.VOID_FILTER]);
             contentDelimitation = true;
         }
 
         // Sendfile support
-        if (endpoint.getUseSendfile()) {
+        if (protocol.getUseSendfile()) {
             prepareSendfile(outputFilters);
         }
 
@@ -1154,13 +942,19 @@ public class Http11Processor extends AbstractProcessor {
             }
             String contentLanguage = response.getContentLanguage();
             if (contentLanguage != null) {
-                headers.setValue("Content-Language").setString(contentLanguage);
+                headers.setValue("Content-Language")
+                    .setString(contentLanguage);
             }
         }
 
         long contentLength = response.getContentLengthLong();
         boolean connectionClosePresent = isConnectionToken(headers, Constants.CLOSE);
-        if (contentLength != -1) {
+        if (http11 && response.getTrailerFields() != null) {
+            // If trailer fields are set, always use chunking
+            outputBuffer.addActiveFilter(outputFilters[Constants.CHUNKED_FILTER]);
+            contentDelimitation = true;
+            headers.addValue(Constants.TRANSFERENCODING).setString(Constants.CHUNKED);
+        } else if (contentLength != -1) {
             headers.setValue("Content-Length").setLong(contentLength);
             outputBuffer.addActiveFilter(outputFilters[Constants.IDENTITY_FILTER]);
             contentDelimitation = true;
@@ -1183,7 +977,8 @@ public class Http11Processor extends AbstractProcessor {
         // Add date header unless application has already set one (e.g. in a
         // Caching Filter)
         if (headers.getValue("Date") == null) {
-            headers.addValue("Date").setString(FastHttpDateFormat.getCurrentDate());
+            headers.addValue("Date").setString(
+                    FastHttpDateFormat.getCurrentDate());
         }
 
         // FIXME: Add transfer encoding header
@@ -1191,14 +986,15 @@ public class Http11Processor extends AbstractProcessor {
         if ((entityBody) && (!contentDelimitation) || connectionClosePresent) {
             // Disable keep-alive if:
             // - there is a response body but way for the client to determine
-            // the content length information; or
+            //   the content length information; or
             // - there is a "connection: close" header present
             // This will cause the "connection: close" header to be added if it
             // is not already present.
             keepAlive = false;
         }
 
-        // This may disable keep-alive so check before working out the Connection header
+        // This may disabled keep-alive to check before working out the
+        // Connection header.
         checkExpectationAndResponseStatus();
 
         // This may disable keep-alive if there is more body to swallow
@@ -1213,7 +1009,8 @@ public class Http11Processor extends AbstractProcessor {
         if (!keepAlive) {
             // Avoid adding the close header twice
             if (!connectionClosePresent) {
-                headers.addValue(Constants.CONNECTION).setString(Constants.CLOSE);
+                headers.addValue(Constants.CONNECTION).setString(
+                        Constants.CLOSE);
             }
         } else if (!getErrorState().isError()) {
             if (!http11) {
@@ -1221,8 +1018,8 @@ public class Http11Processor extends AbstractProcessor {
             }
 
             if (protocol.getUseKeepAliveResponseHeader()) {
-                boolean connectionKeepAlivePresent = isConnectionToken(request.getMimeHeaders(),
-                        Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
+                boolean connectionKeepAlivePresent =
+                    isConnectionToken(request.getMimeHeaders(), Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
 
                 if (connectionKeepAlivePresent) {
                     int keepAliveTimeout = protocol.getKeepAliveTimeout();
@@ -1236,11 +1033,10 @@ public class Http11Processor extends AbstractProcessor {
                             // else create the header
                             MessageBytes connectionHeaderValue = headers.getValue(Constants.CONNECTION);
                             if (connectionHeaderValue == null) {
-                                headers.addValue(Constants.CONNECTION)
-                                        .setString(Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
+                                headers.addValue(Constants.CONNECTION).setString(Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
                             } else {
-                                connectionHeaderValue.setString(connectionHeaderValue.getString() + ", " +
-                                        Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
+                                connectionHeaderValue.setString(
+                                        connectionHeaderValue.getString() + ", " + Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
                             }
                         }
                     }
@@ -1269,8 +1065,8 @@ public class Http11Processor extends AbstractProcessor {
                     outputBuffer.sendHeader(headers.getName(i), headers.getValue(i));
                 } catch (IllegalArgumentException iae) {
                     // Log the problematic header
-                    log.warn(sm.getString("http11processor.response.invalidHeader", headers.getName(i),
-                            headers.getValue(i)), iae);
+                    log.warn(sm.getString("http11processor.response.invalidHeader",
+                            headers.getName(i), headers.getValue(i)), iae);
                     // Remove the problematic header
                     headers.removeHeader(i);
                     size--;
@@ -1305,23 +1101,27 @@ public class Http11Processor extends AbstractProcessor {
 
 
     private void prepareSendfile(OutputFilter[] outputFilters) {
-        String fileName = (String) request.getAttribute(org.apache.coyote.Constants.SENDFILE_FILENAME_ATTR);
+        String fileName = (String) request.getAttribute(
+                org.apache.coyote.Constants.SENDFILE_FILENAME_ATTR);
         if (fileName == null) {
             sendfileData = null;
         } else {
             // No entity body sent here
             outputBuffer.addActiveFilter(outputFilters[Constants.VOID_FILTER]);
             contentDelimitation = true;
-            long pos = ((Long) request.getAttribute(org.apache.coyote.Constants.SENDFILE_FILE_START_ATTR)).longValue();
-            long end = ((Long) request.getAttribute(org.apache.coyote.Constants.SENDFILE_FILE_END_ATTR)).longValue();
+            long pos = ((Long) request.getAttribute(
+                    org.apache.coyote.Constants.SENDFILE_FILE_START_ATTR)).longValue();
+            long end = ((Long) request.getAttribute(
+                    org.apache.coyote.Constants.SENDFILE_FILE_END_ATTR)).longValue();
             sendfileData = socketWrapper.createSendfileData(fileName, pos, end - pos);
         }
     }
 
 
     /*
-     * Note: populateHost() is not over-ridden. request.serverName() will be set to return the default host name by the
-     * Mapper.
+     * Note: populateHost() is not over-ridden.
+     *       request.serverName() will be set to return the default host name by
+     *       the Mapper.
      */
 
 
@@ -1359,7 +1159,7 @@ public class Http11Processor extends AbstractProcessor {
 
     @Override
     protected SocketState dispatchEndRequest() {
-        if (!keepAlive || endpoint.isPaused()) {
+        if (!keepAlive || protocol.isPaused()) {
             return SocketState.CLOSED;
         } else {
             endRequest();
@@ -1381,8 +1181,9 @@ public class Http11Processor extends AbstractProcessor {
 
 
     /*
-     * No more input will be passed to the application. Remaining input will be swallowed or the connection dropped
-     * depending on the error and expectation status.
+     * No more input will be passed to the application. Remaining input will be
+     * swallowed or the connection dropped depending on the error and
+     * expectation status.
      */
     private void endRequest() {
         if (getErrorState().isError()) {
@@ -1498,18 +1299,20 @@ public class Http11Processor extends AbstractProcessor {
             // Consume and buffer the request body, so that it does not
             // interfere with the client's handshake messages
             InputFilter[] inputFilters = inputBuffer.getFilters();
-            ((BufferedInputFilter) inputFilters[Constants.BUFFERED_FILTER]).setLimit(maxSavePostSize);
+            ((BufferedInputFilter) inputFilters[Constants.BUFFERED_FILTER]).setLimit(
+                    protocol.getMaxSavePostSize());
             inputBuffer.addActiveFilter(inputFilters[Constants.BUFFERED_FILTER]);
 
             /*
-             * Outside the try/catch because we want I/O errors during renegotiation to be thrown for the caller to
-             * handle since they will be fatal to the connection.
+             * Outside the try/catch because we want I/O errors during
+             * renegotiation to be thrown for the caller to handle since they
+             * will be fatal to the connection.
              */
             socketWrapper.doClientAuth(sslSupport);
             try {
                 /*
-                 * Errors processing the cert chain do not affect the client connection so they can be logged and
-                 * swallowed here.
+                 * Errors processing the cert chain do not affect the client
+                 * connection so they can be logged and swallowed here.
                  */
                 Object sslO = sslSupport.getPeerCertificateChain();
                 if (sslO != null) {
@@ -1566,6 +1369,34 @@ public class Http11Processor extends AbstractProcessor {
     }
 
 
+    @Override
+    protected boolean isTrailerFieldsReady() {
+        if (inputBuffer.isChunking()) {
+            return inputBuffer.isFinished();
+        } else {
+            return true;
+        }
+    }
+
+
+    @Override
+    protected boolean isTrailerFieldsSupported() {
+        // Request must be HTTP/1.1 to support trailer fields
+        if (!http11) {
+            return false;
+        }
+
+        // If the response is not yet committed, chunked encoding can be used
+        // and the trailer fields sent
+        if (!response.isCommitted()) {
+            return true;
+        }
+
+        // Response has been committed - need to see if chunked is being used
+        return outputBuffer.isChunking();
+    }
+
+
     /**
      * Trigger sendfile processing if required.
      *
@@ -1588,15 +1419,15 @@ public class Http11Processor extends AbstractProcessor {
             }
             result = socketWrapper.processSendfile(sendfileData);
             switch (result) {
-                case ERROR:
-                    // Write failed
-                    if (log.isDebugEnabled()) {
-                        log.debug(sm.getString("http11processor.sendfile.error"));
-                    }
-                    setErrorState(ErrorState.CLOSE_CONNECTION_NOW, null);
-                    //$FALL-THROUGH$
-                default:
-                    sendfileData = null;
+            case ERROR:
+                // Write failed
+                if (log.isDebugEnabled()) {
+                    log.debug(sm.getString("http11processor.sendfile.error"));
+                }
+                setErrorState(ErrorState.CLOSE_CONNECTION_NOW, null);
+                //$FALL-THROUGH$
+            default:
+                sendfileData = null;
             }
         }
         return result;
@@ -1612,6 +1443,7 @@ public class Http11Processor extends AbstractProcessor {
         upgradeToken = null;
         socketWrapper = null;
         sendfileData = null;
+        sslSupport = null;
     }
 
 
